/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpTimerManager.h>



unsigned int UpTimerManager::catchUp() {
	// primero se agregan los nuuevos
	mx.lock();
	while(!tAddList.empty()) {
		tSet.insert(tAddList.front());
		tAddList.pop_front();
	}
	mx.unlock();
	// catchup loop principal
	// cannot accept insertions in tSet during the catchup loop
	// it is also dangerous to accept them between each iteration of the loop
	UpTimerSet::iterator it,auxit;
	unsigned int timerCount(0);
	UpTimer* timer;
	nowTime.now();
	it=tSet.begin();
	while( it!=tSet.end() ) {
		if( **it>nowTime ) break;
		timer = *it;
		auxit = it++;
		tSet.erase(auxit);
	   if( timer->adminAction(nowTime) ) tSet.insert(timer);
		else delete timer;
    }
	return timerCount;
}



#include <iostream>
void* UpTimerManager::run() {
	// usleep needs an unsigned int
 	unsigned int totalMicroSeconds=tickInterval.getTotalMicroSeconds();
	while(1){
    	catchUp();
    	usleep(totalMicroSeconds);
		testCancel(); 
    }
    exit((void*)"coming out of the timeLine thread");
	 // the following is a dummy instruction, just for the compiler warning
	 return (void*)"coming out of the timeLine thread";
}





UpTimerManager::~UpTimerManager() {
	cancel(); // cancel independent timers thread
	for(UpTimerSet::iterator it=tSet.begin(); it!=tSet.end(); it++)
		{ delete *it; }
	mx.lock();
	while(!tAddList.empty())
		{ delete tAddList.front(); tAddList.pop_front(); }
	mx.unlock();
}

